from datetime import datetime
from .switch import Switch

class Insight(Switch):

    def __repr__(self):
        return '<WeMo Insight "{name}">'.format(name=self.name)

    @property
    def insight_params(self):
        params = self.insight.GetInsightParams().get('InsightParams')
        (
            state,  # 0 if off, 1 if on, 8 if on but load is off
            lastchange,
            onfor,  # seconds
            ontoday,  # seconds
            ontotal,  # seconds
            timeperiod,  # The period over which averages are calculated
            _x,  # This one is always 19 for me; what is it?
            currentmw,
            todaymw,
            totalmw,
            powerthreshold
        ) = params.split('|')
        return {'state': state,
                'lastchange': datetime.fromtimestamp(int(lastchange)),
                'onfor': int(onfor),
                'ontoday': int(ontoday),
                'ontotal': int(ontotal),
                'todaymw': int(float(todaymw)),
                'totalmw': int(float(totalmw)),
                'currentpower': int(float(currentmw))}

    @property
    def today_kwh(self):
        return self.insight_params['todaymw'] * 1.6666667e-8

    @property
    def current_power(self):
        """
        Returns the current power usage in mW.
        """
        return self.insight_params['currentpower']

    @property
    def today_on_time(self):
        return self.insight_params['ontoday']

    @property
    def on_for(self):
        return self.insight_params['onfor']

    @property
    def last_change(self):
        return self.insight_params['lastchange']

    @property
    def today_standby_time(self):
        return self.insight_params['ontoday']

    @property
    def ontotal(self):
        return self.insight_params['ontotal']

    @property
    def totalmw(self):
        return self.insight_params['totalmw']



from functools import wraps
import re
import socket
import struct
import time

import gevent
import requests


def tz_hours():
    delta = time.localtime().tm_hour - time.gmtime().tm_hour
    sign = '-' if delta < 0 else ''
    return "%s%02d.00" % (sign, abs(delta))


def is_dst():
    return 1 if time.localtime().tm_isdst else 0


def get_timesync():
    timesync = """
<?xml version="1.0" encoding="utf-8"?>
<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
 <s:Body>
  <u:TimeSync xmlns:u="urn:Belkin:service:timesync:1">
   <UTC>{utc}</UTC>
   <TimeZone>{tz}</TimeZone>
   <dst>{dst}</dst>
   <DstSupported>{dstsupported}</DstSupported>
  </u:TimeSync>
 </s:Body>
</s:Envelope>""".format(
        utc=int(time.time()),
        tz=tz_hours(),
        dst=is_dst(),
        dstsupported=is_dst()).strip()
    return timesync


def get_ip_address():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('1.2.3.4', 9))
        return s.getsockname()[0]
    except socket.error:
        return None
    finally:
        del s


def matcher(match_string):
    pattern = re.compile('.*?'.join(re.escape(c) for c in match_string.lower()))
    def matches(s):
        return pattern.search(s.lower()) is not None
    return matches


# This is pretty arbitrary. I'm choosing, for no real reason, the length of
# a subscription.
_RETRIES = 1801/60


def get_retries():
    return _RETRIES


def retry_with_delay(f, delay=60):
    """
    Retry the wrapped requests.request function in case of ConnectionError.
    Optionally limit the number of retries or set the delay between retries.
    """
    @wraps(f)
    def inner(*args, **kwargs):
        kwargs['timeout'] = 5
        remaining = get_retries() + 1
        while remaining:
            remaining -= 1
            try:
                return f(*args, **kwargs)
            except (requests.ConnectionError, requests.Timeout):
                if not remaining:
                    raise
                gevent.sleep(delay)
    return inner


requests_get = retry_with_delay(requests.get)
requests_post = retry_with_delay(requests.post)
requests_request = retry_with_delay(requests.request)



from ouimeaux.device import Device

from xml.etree import cElementTree as et


class Bridge(Device):
    Lights = {}

#    def __init__(self, address):
#        pass
        #self.bridge_get_lights()

    def __repr__(self):
        self.bridge_get_lights()
#        for light in self.Lights:
#            print 'Light: {name}, State: {state}'.format(name=self.light_name(light), state=self.light_get_state(light).get('state'))
        return '<WeMo Bridge "{name}" Lights: {LightCount} >'.format(name=self.name, LightCount=len(self.Lights))

    def bridge_get_lights(self):
        UDN = self.basicevent.GetMacAddr().get('PluginUDN')
        endDevices = self.bridge.GetEndDevices(DevUDN=UDN,ReqListType='PAIRED_LIST').get('DeviceLists')
        endDevicesList = et.fromstring(endDevices)
        for light in endDevicesList[0][1]:
#            if self.light_name(light) in [ self.light_name(x) for x in self.Lights ]:
            if self.light_name(light) in self.Lights:
                pass
            else:
                #self.Lights.append(light)
                self.Lights[self.light_name(light)] = light
        return self.Lights

    def light_attributes(self, light):
        return {
            'devIndex' : light[0].text,
            'devID' : light[1].text,
            'name' : light[2].text,
            'iconvalue' : light[3].text,
            'firmware' : light[4].text,
            'capabilities' : light[5].text,
            'state' : light[6].text,
            'manufacturer' : light[7].text,
            'model' : light[8].text,
            'certified' : light[9].text
        }

    def light_name(self, light):
        return self.light_attributes(light).get('name')

    def light_get_id(self, light):
        return self.light_attributes(light).get('devID')

    def light_get_state(self, light):
        (
            state, # 0 (off) or 1 (on)
            dim # 0-255 dark to bright
        ) = self.light_attributes(light).get('state').split(':', 1)[0].split(',',1)
        return {
            'state' : state,
            'dim' : dim
        }

    def light_set_state(self, light, state=None, dim=None):
        if state == None:
            state = self.light_get_state(light).get('state')
        if dim == None:
            state = self.light_get_state(light).get('dim')

        sendState = '&lt;?xml version=&quot;1.0&quot; encoding=&quot;UTF-8&quot;?&gt;&lt;DeviceStatus&gt;&lt;IsGroupAction&gt;NO&lt;/IsGroupAction&gt;&lt;DeviceID available=&quot;YES&quot;&gt;{devID}&lt;/DeviceID&gt;&lt;CapabilityID&gt;10006,10008&lt;/CapabilityID&gt;&lt;CapabilityValue&gt;{state},{dim}&lt;/CapabilityValue&gt;&lt;/DeviceStatus&gt;'.format(devID=self.light_get_id(light),state=state,dim=dim)

        return self.bridge.SetDeviceStatus(DeviceStatusList=sendState)




import os
import json
import gevent
from flask import Flask, request, Response
from flask import render_template, send_from_directory, url_for
from flask import send_file, make_response, abort
from flask.ext.restful import reqparse, abort, Api, Resource


from ouimeaux.signals import statechange
from ouimeaux.device.switch import Switch
from ouimeaux.device.insight import Insight
from ouimeaux.device.maker import Maker
from ouimeaux.environment import Environment, UnknownDevice
from socketio import socketio_manage
from socketio.namespace import BaseNamespace
from socketio.mixins import BroadcastMixin

here = lambda *x: os.path.join(os.path.dirname(__file__), *x)


app = Flask(__name__)
api = Api(app)

ENV = None


def initialize(bind=None):
    global ENV
    if ENV is None:
        ENV = Environment(with_cache=False, bind = bind)
        ENV.start()
        gevent.spawn(ENV.discover, 10)


def serialize(device):
    if isinstance(device, Insight):
      return {'name': device.name,
              'type': device.__class__.__name__,
              'serialnumber': device.serialnumber,
              'state': device.get_state(),
              'model': device.model,
              'host': device.host,
              'lastchange': str(device.last_change),
              'onfor': device.on_for,
              'ontoday': device.today_on_time,
              'ontotal': device.ontotal,
              'todaymw': device.today_kwh,
              'totalmw': device.totalmw,
              'currentpower': device.current_power
              }
    elif isinstance(device, Maker):
       return {'name': device.name,
              'type': device.__class__.__name__,
              'serialnumber': device.serialnumber,
              'state': device.get_state(),
              'model': device.model,
              'host': device.host,
              'hassensor' : device.has_sensor,
              'switchmode' : device.switch_mode,
              'sensor' : device.sensor_state
              }
    return {'name': device.name,
            'type': device.__class__.__name__,
            'serialnumber': device.serialnumber,
            'state': device.get_state(),
            'model': device.model,
            'host': device.host}


def get_device(name, should_abort=True):
    try:
        return ENV.get(name)
    except UnknownDevice:
        if not should_abort:
            raise
        abort(404, error='No device matching {}'.format(name))


# First, the REST API
class EnvironmentResource(Resource):

    def get(self):
        result = {}
        for dev in ENV:
            result[dev.name] = serialize(dev)
        return result

    def post(self):
        seconds = (request.json or {}).get('seconds', (
            request.values or {}).get('seconds', 5))
        ENV.discover(int(seconds))
        return self.get()


class DeviceResource(Resource):

    def get(self, name):
        return serialize(get_device(name))

    def post(self, name):
        dev = get_device(name)
        if not isinstance(dev, Switch):
            abort(405, error='Only switches can have their state changed')
        action = (request.json or {}).get('state', (
            request.values or {}).get('state', 'toggle'))
        if action not in ('on', 'off', 'toggle', 'blink'):
            abort(400, error='{} is not a valid state'.format(action))
        if action == 'blink':
            delay = (request.json or {}).get('delay', (
                request.values or {}).get('delay', '1'))
            getattr(dev, action)(delay=int(delay))
        else:
            getattr(dev, action)()
        return serialize(dev)


api.add_resource(EnvironmentResource, '/api/environment')
api.add_resource(DeviceResource, '/api/device/<string:name>')


class SocketNamespace(BaseNamespace):

    def update_state(self, sender, **kwargs):
        data = serialize(sender)
        data['state'] = kwargs.get('state', data['state'])
        self.emit("send:devicestate", data)

    def on_statechange(self, data):
        ENV.get(data['name']).set_state(data['state'])

    def on_join(self, data):
        statechange.connect(self.update_state,
                            unique=False,
                            dispatch_uid=id(self))
        for device in ENV:
            self.update_state(device)

    def __del__(self):
        statechange.disconnect(dispatch_uid=id(self))


# Now for the WebSocket api
@app.route("/socket.io/<path:path>")
def run_socketio(**kwargs):
    socketio_manage(request.environ, {'': SocketNamespace})


# routing for basic pages (pass routing onto the Angular app)
@app.route('/')
def basic_pages(**kwargs):
    return make_response(open(here('templates/index.html')).read())


# special file handlers and error handlers
@app.route('/favicon.ico')
def favicon():
    return send_from_directory(os.path.join(app.root_path, 'static'),
                               'img/favicon.ico')

@app.errorhandler(404)
def page_not_found(e):
    return render_template('404.html'), 404

app.config.from_object('ouimeaux.server.settings')
app.url_map.strict_slashes = False

if __name__ == "__main__":
    app.run()




import random
import datetime
import time
import ouimeaux
from ouimeaux.environment import Environment

# http://pydoc.net/Python/ouimeaux/0.7.3/ouimeaux.examples.watch/
if __name__ == "__main__":
    print ""
    print "WeMo Randomizer"
    print "---------------"
    env = Environment()
    # TODO: run from 10am to 10pm
    try:
        env.start()
        env.discover(100)
        print env.list_switches()
        print env.list_motions()
        print "---------------"
        while True:
            # http://stackoverflow.com/questions/306400/how-do-i-randomly-select-an-item-from-a-list-using-python
            switchRND = env.get_switch( random.choice( env.list_switches() ) )
            print switchRND
            switchRND.toggle()
            env.wait(90)
        
    except (KeyboardInterrupt, SystemExit):
        print "---------------"
        print "Goodbye!"
        print "---------------"
        # Turn off all switches
        for switch in ( env.list_switches() ):
            print "Turning Off: " + switch
            env.get_switch( switch ).off()



from datetime import datetime
from ouimeaux.device import Device
from xml.etree import cElementTree as et


class Maker(Device):

    def __repr__(self):
        return '<WeMo Maker "{name}">'.format(name=self.name)
        
    def set_state(self, state):
        """
        Set the state of this device to on or off.
        """
        self.basicevent.SetBinaryState(BinaryState=int(state))
        self._state = int(state)

    def off(self):
        """
        Turn this device off. If already off, will return "Error".
        """
        return self.set_state(0)

    def on(self):
        """
        Turn this device on. If already on, will return "Error".
        """
        return self.set_state(1)

    @property
    def maker_attribs(self):
        makerresp = self.deviceevent.GetAttributes().get('attributeList')
        makerresp = "<attributes>" + makerresp + "</attributes>"
        makerresp = makerresp.replace("&gt;",">")
        makerresp = makerresp.replace("&lt;","<")
        attributes = et.fromstring(makerresp)
        for attribute in attributes:
            if attribute[0].text == "Sensor":
            	sensorstate = attribute[1].text
            elif attribute[0].text == "SwitchMode":
            	switchmode = attribute[1].text
            elif attribute[0].text == "SensorPresent":
            	hassensor = attribute[1].text
        return { 'sensorstate' : int(sensorstate),
        		 'switchmode' : int(switchmode),
        		 'hassensor' : int(hassensor)}

    @property
    def sensor_state(self):
        return self.maker_attribs['sensorstate']
        
    @property
    def switch_mode(self):
    	return self.maker_attribs['switchmode']
    
    @property
    def has_sensor(self):
    	return self.maker_attribs['hassensor']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

here = lambda *a: os.path.join(os.path.dirname(__file__), *a)


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open(here('README.rst')).read()
history = open(here('HISTORY.rst')).read().replace('.. :changelog:', '')
requirements = [x.strip() for x in open(here('requirements.txt')).readlines()]

setup(
    name='ouimeaux',
    version='0.7.9',
    description='Open source control for Belkin WeMo devices',
    long_description=readme + '\n\n' + history,
    author='Ian McCracken',
    author_email='ian.mccracken@gmail.com',
    url='https://github.com/iancmcc/ouimeaux',
    packages=[
        'ouimeaux',
    ],
    package_dir={'ouimeaux': 'ouimeaux'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='ouimeaux',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Topic :: Home Automation',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    entry_points={
        'console_scripts': [
            'wemo = ouimeaux.cli:wemo'
        ]
    },
    test_suite='tests',
)


#!/usr/bin/env python
import argparse
import sys

from ouimeaux.environment import Environment
from ouimeaux.utils import matcher
from ouimeaux.signals import receiver, statechange, devicefound


def mainloop(name):
    matches = matcher(name)

    @receiver(devicefound)
    def found(sender, **kwargs):
        if matches(sender.name):
            print "Found device:", sender.name

    @receiver(statechange)
    def motion(sender, **kwargs):
        if matches(sender.name):
            print "{} state is {state}".format(
                sender.name, state="on" if kwargs.get('state') else "off")

    env = Environment(with_cache=False)
    try:
        env.start()
        env.discover(10)
        env.wait()
    except (KeyboardInterrupt, SystemExit):
        print "Goodbye!"
        sys.exit(0)


if __name__ == "__main__":
    parser = argparse.ArgumentParser("Motion notifier")
    parser.add_argument("name", metavar="NAME",
                        help="Name (fuzzy matchable)"
                             " of the Motion to detect")
    args = parser.parse_args()
    mainloop(args.name)
